#!/usr/bin/perl

# After running this, can look at /admin/snmptrapexception/list/all to see how it looks

use strict;
use warnings;

use Test::More qw(no_plan);

use FindBin qw($Bin);
use lib "$Bin/lib", "$Bin/../etc", "$Bin/../lib";
use File::Compare;
use Opsview;
use Opsview::Snmptrapexception;

use_ok("Opsview::Test", qw(opsview));

chdir("$Bin/..");

my $exceptions = "t/var/exceptions.generated";

# Temporary fix for nagconfgen warnings on hudson - not sure why
$ENV{DBIC_UTF8COLUMNS_OK}=1;

my @output = `bin/nagconfgen.pl -s t/var/snmptraps.generated 2>&1`;
my @expected = ( "WARNING: Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host\n", "WARNING: IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3\n" );
is_deeply( \@output, \@expected, "Warnings as expected" ) || diag("Full output: ".join("", @output));

my $rc = system("bin/nagconfgen.pl -s t/var/snmptraps.generated");
is( $rc, 0, "trap configuration generated" );

# Ignore white space
ok( system("diff -w t/var/snmptraps.generated t/var/snmptraps.expected") == 0, "Difference between t/var/snmptraps.generated and snmptraps.expected" );

unlink $exceptions if -e $exceptions;
open F, "cat t/var/snmptraps | bin/snmptrap2nagios -r -c t/var/snmptraps.expected -e $exceptions |";
@output = <F>;
close F;

@expected = ("SERVICE CHECK: cisco Coldstart 1 Device coldstarted\n", "SERVICE CHECK: cisco Coldstart 1 Device coldstarted\n");

is_deeply( \@output, \@expected, "Output as expected" );

open F, "$exceptions" or die "Cannot open $exceptions";
@output = ();
{
    local $/ = "#---next trap---#\n";
    while (<F>) {
        chomp;
        s/^(\d+)\n//;
        cmp_ok( time, "<=", $1 + 5, "Time within 5 seconds of now" );
        push @output, $_;
    }
}
close F;

@expected = (
    '2 cisco
192.168.10.20
192.168.10.20
SNMPv2-MIB::sysUpTime.0 4:20:49:47.73
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-SMI::enterprises.9.9.43.2.0.1
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.3.45 1
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.4.45 2
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.5.45 3
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 192.168.10.20
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"',
    '6 cisco 81,82
192.168.10.20
192.168.10.20
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-MIB::coldStart
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-SMI::enterprises.9.2.1.2.0 "reload"
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 172.16.0.1
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"
SNMPv2-MIB::snmpTrapEnterprise.0 SNMPv2-SMI::enterprises.9.1.186',
    '5
192.168.10.21
192.168.10.21
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-MIB::coldStart
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-SMI::enterprises.9.2.1.2.0 "reload"
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 192.168.10.21
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"
SNMPv2-MIB::snmpTrapEnterprise.0 SNMPv2-SMI::enterprises.9.1.186',
    '1
192.168.10.20
192.168.10.20
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-MIB::snmpTrapMalformedOID.0 Malformed',
    # Check for exception here from solarwinds forwarded trap. If solarwinds code not in snmptrap2nagios,
    # then the exception would be different (IP not found)
    '6 cisco 81,82
192.168.100.100
192.168.100.100
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-MIB::coldStart
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-SMI::enterprises.9.2.1.2.0 "reload"
SOLARWINDS-TRAPS::nodeIP 192.168.10.20
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 172.16.0.1
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "solarwindsforwarding"
SNMPv2-MIB::snmpTrapEnterprise.0 SNMPv2-SMI::enterprises.9.1.186'
);

is_deeply( \@output, \@expected, "Trap exceptions at $exceptions correct" );

use Test::DatabaseRow;
local $Test::DatabaseRow::dbh = Opsview::Snmptrapexception->db_Main;

# Now test it gets into exceptions table okay
Opsview::Snmptrapexception->retrieve_all->delete_all;
is( Opsview::Snmptrapexception->count_all,                0, "Table empty" );
is( system("bin/update_snmptrap_exceptions $exceptions"), 0, "Run update of snmptraps successfully" );

row_ok(
    sql   => "SELECT count(*) as count from snmptrapexceptions",
    tests => [ count => 5 ],
    label => "Should be 5 exceptions",
);

row_ok(
    table   => "snmptrapexceptions",
    where   => [ reason => 2 ],
    results => 1,
    tests   => [ hostip => "192.168.10.20", trapname => "SNMPv2-SMI::enterprises.9.9.43.2.0.1", hostname => "cisco", trapdebug => undef, arg1 => undef ],
    label   => "Trapname not fully translated",
);

row_ok(
    table   => "snmptrapexceptions",
    where   => [ reason => 6 ],
    results => 2,
    tests   => [ hostip => "192.168.10.20", trapname => "SNMPv2-MIB::coldStart", hostname => "cisco", trapdebug => undef, arg1 => "81,82" ],
    label   => "2 servicechecks failed",
);

row_ok(
    table   => "snmptrapexceptions",
    where   => [ reason => 5 ],
    results => 1,
    tests   => [ hostip => "192.168.10.21", trapname => "SNMPv2-MIB::coldStart", hostname => undef, trapdebug => undef, arg1 => undef ],
    label   => "Not monitoring this hostip",
);

row_ok(
    table   => "snmptrapexceptions",
    where   => [ reason => 1 ],
    results => 1,
    tests   => [ hostip => "192.168.10.20", trapname => undef, hostname => undef, trapdebug => undef, arg1 => undef ],
    label   => "No trapname",
);

# Test tracing works
unlink "var/traps.db";
unlink $exceptions if -e $exceptions;
open F, "cat t/var/snmptraps.for_tracing | bin/snmptrap2nagios -r -f -c t/var/snmptraps.expected -e $exceptions |";
@output = <F>;
close F;

ok( -e "var/traps.db", "Traps db created" );

# Test tracing still works if the file is 0 bytes (occurred on opsview.hub)
unlink "var/traps.db";
system("touch var/traps.db") == 0 or die "Error touching file: $!";
is( -s "var/traps.db", 0, "File exists and is 0 bytes" );
open F, "cat t/var/snmptraps.for_tracing | bin/snmptrap2nagios -r -f -c t/var/snmptraps.expected -e $exceptions |";
@output = <F>;
close F;
cmp_ok( -s "var/traps.db", ">", 0, "Now bigger than 0" );

my $tracing_dbh = DBI->connect("dbi:SQLite:dbname=var/traps.db") or die "Cannot connect to db";
local $Test::DatabaseRow::dbh = $tracing_dbh;
row_ok(
    sql   => "SELECT count(*) as count from tracing",
    tests => [ count => 2 ],
    label => "Should be 2 traps saved",
);

row_ok(
    table   => "tracing",
    where   => [ hostname => "cisco2" ],
    results => 1,
    tests   => [
        trap => qq{192.168.10.22
192.168.10.22
SNMPv2-MIB::sysUpTime.0 4:20:49:47.73
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-SMI::enterprises.9.9.43.2.0.1
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.3.45 1
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.4.45 2
SNMPv2-SMI::enterprises.9.9.43.1.1.6.1.5.45 3
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 192.168.10.20
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"}
    ],
    label => "Trap as expected",
);

row_ok(
    table   => "tracing",
    where   => [ hostname => "cisco1" ],
    results => 1,
    tests   => [
        trap => qq{192.168.10.23
192.168.10.23
SNMPv2-MIB::sysUpTime.0 0:0:00:22.66
SNMPv2-MIB::snmpTrapOID.0 SNMPv2-MIB::coldStart
SNMPv2-SMI::enterprises.9.2.1.2.0 "reload"
SNMP-COMMUNITY-MIB::snmpTrapAddress.0 172.16.0.1
SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"
SNMPv2-MIB::snmpTrapEnterprise.0 SNMPv2-SMI::enterprises.9.1.186}
    ],
    label => "Trap as expected",
);
